<?php

namespace Drupal\commerce_yandex_metrika_ecommerce;

/**
 * Creating script with data for Yandex Metrika.
 */
class Script {

  /**
   * Constructor.
   *
   * @param array $items
   *   Items for Yandex Metrika: <br>
   *    content - item content with data for Yandex Metrika: <br>
   *      type - 'product' or 'order', <br>
   *
   *      for product type: <br>
   *      currency_code - currency code, <br>
   *      action        - 'add' or 'remove' product in/from cart, <br>
   *      id            - product id, <br>
   *      title         - product title, <br>
   *      quantity      - product quantity, <br>
   *      price         - product price without the decimal, <br>
   *
   *      for order type: <br>
   *        id       - order id, <br>
   *        total    - order total price without the decimal, <br>
   *        products - order products: <br>
   *          id       - product id, <br>
   *          title    - product title, <br>
   *          quantity - product quantity, <br>
   *          price    - product price without the decimal.
   */
  public function __construct(array $items) {
    $this->items = $items;
  }

  /**
   * Show script.
   */
  public function show() { ?>
    <script type="text/javascript">
      Drupal.behaviors.commerce_yandex_metrika_ecommerce = {
        attach: function() {
          window.dataLayer = window.dataLayer || [];
          let aItems = (<?php echo json_encode($this->getItems())?>);
          console.log(aItems);
          for(let i = 0; i < aItems.length; i++) {
            window.dataLayer.push(aItems[i]);
          }
        }
      }
    </script>
  <?php }

  /**
   * Items for Yandex Metrika.
   *
   * Format: <br>
   *  content - item content with data for Yandex Metrika: <br>
   *    type - 'product' or 'order', <br>.
   *
   *    For product type: <br>
   *    currency_code - currency code, <br>
   *    action        - 'add' or 'remove' product in/from cart, <br>
   *    id            - product id, <br>
   *    title         - product title, <br>
   *    quantity      - product quantity, <br>
   *    price         - product price without the decimal, <br>
   *
   *    for order type: <br>
   *      id       - order id, <br>
   *      total    - order total price without the decimal, <br>
   *      products - order products: <br>
   *        id       - product id, <br>
   *        title    - product title, <br>
   *        quantity - product quantity, <br>
   *        price    - product price without the decimal.
   *
   * @var array
   */
  private $items;

  /**
   * Get items array in Yandex Metrika format.
   *
   * @see https://yandex.ru/support/metrika/data/e-commerce.html
   *
   * @return array
   *   Items in Yandex Metrika format.
   */
  private function getItems() {
    $checkout_goal = variable_get('commerce_yandex_metrika_ecommerce_checkout_complete_goal_id');
    $items = [];
    foreach ($this->items as $item) {
      if ($item->content->type == 'product') {
        $items[] = [
          'ecommerce' => [
            'currencyCode' => $item->content->currency_code,
            $item->content->action => [
              'products' => [[
                'id' => $item->content->id,
                'name' => $item->content->title,
                'quantity' => $item->content->quantity,
                'price' => $item->content->price,
              ],
              ],
            ],
          ],
        ];
      }
      elseif ($item->content->type == 'order') {
        $products = [];
        foreach ($item->content->products as $product) {
          $products[] = [
            'id' => $product->id,
            'name' => $product->title,
            'quantity' => $product->quantity,
            'price' => $product->price,
          ];
        }
        $order_item = [
          'ecommerce' => [
            'purchase' => [
              'actionField' => [
                'id' => $item->content->id,
                'revenue' => $item->content->total,
              ],
              'products' => $products,
            ],
          ],
        ];
        if($checkout_goal) {
          $order_item['ecommerce']['purchase']['actionField']['goal_id'] = $checkout_goal;
        }
        $items[] = $order_item;
      }
    }
    return $items;
  }

}
